<?php

namespace Drupal\dynamic_yield;

/**
 * Provides batch queue callbacks for Dynamic Yield.
 */
class DynamicYieldBatchQueueUpdates {

  /**
   * Batch operation callback.
   *
   * @param int $batchId
   *   The index of this batch operation.
   * @param int $total
   *   The total number if items in all batches.
   * @param \Drupal\dynamic_yield\DynamicYieldUpdateItem[] $items
   *   A set of data to add to the queue.
   * @param array $context
   *   Persistent data.
   */
  public static function process(int $batchId, int $total, array $items, array &$context): void {
    /** @var \Drupal\dynamic_yield\Services\DynamicYieldUpdateCollectionInterface $priorityQueue */
    $priorityQueue = \Drupal::service('dynamic_yield.updates');
    // Initiate data on the first step.
    if (!isset($context['sandbox']['progress'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['max'] = $total;
    }

    foreach ($items as $item) {
      $priorityQueue->add($item);
    }
    $count = count($items);
    $context['sandbox']['progress'] += $count;
    $context['results']['added'] = $context['sandbox']['progress'];
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    $percent = round($context['sandbox']['finished'] * 100) . '%';
    $context['message'] = t('[@percent] Sent @count items to the DY update queue: @progress of @total items queued.', [
      '@percent' => $percent,
      '@progress' => $context['sandbox']['progress'],
      '@count' => $count,
      '@total' => $total,
    ]);
  }

  /**
   * Batch finish callback.
   *
   * @param bool $success
   *   Did the batch succeed in all operations.
   * @param array $results
   *   Result data.
   * @param array $operations
   *   Any remaining operations.
   */
  public static function finish(bool $success, array $results, array $operations): void {
    $messenger = \Drupal::messenger();
    if ($success) {
      $messenger->addStatus(t('Successfully processed @updated items.', [
        '@updated' => $results['added'],
      ]));
    }
    else {
      // An error occurred.
      // $operations contains the operations that remained unprocessed.
      $error_operation = reset($operations);
      $message = t('An error occurred while processing %error_operation with arguments: @arguments', [
        '%error_operation' => $error_operation[0],
        '@arguments' => print_r($error_operation[1], TRUE),
      ]);
      $messenger->addError($message);
    }
  }

}
